@@ -17,7 +17,7 @@
 char    *version[] = 
 {
     "9wm version 1.1, Copyright (c) 1994 David Hogan", 
-    "w9wm version 0.2, Copyright (c) 2000 Benjamin Drieu", 0,
+    "w9wm version 0.4.1, Copyright (c) 2000 Benjamin Drieu", 0,
 };
 
 Display         *dpy;
@@ -33,6 +33,7 @@ XFontStruct     *font;
 int             nostalgia;
 char            **myargv;
 char            *termprog;
+char            *display = NULL;
 char            *shell;
 Bool            shape;
 int             shape_event;
@@ -41,6 +42,9 @@ int             min_cmaps;
 int             curtime;
 int             debug;
 int             signalled;
+Bool 		click_passes = 0;
+Bool		use_keys = 1;
+int		numvirtuals = 4;
 
 Atom        exit_9wm;
 Atom        restart_9wm;
@@ -84,23 +88,45 @@ char    *argv[];
     for (i = 1; i < argc; i++)
         if (strcmp(argv[i], "-nostalgia") == 0)
             nostalgia++;
-        else if (strcmp(argv[i], "-grey") == 0)
-            background = 1;
         else if (strcmp(argv[i], "-debug") == 0)
             debug++;
+        else if ((strcmp(argv[i], "-display") == 0 || strcmp(argv[i], "-dpy") == 0 ) 
+		 && i+1<argc)
+	  {
+            display = argv[++i];
+	    setenv("DISPLAY", display, 1);
+	  }
+        else if (strcmp(argv[i], "-pass") == 0)
+	  {
+	    click_passes = 1;
+	  }
+        else if (strcmp(argv[i], "-nokeys") == 0)
+	  {
+	    use_keys = 0;
+	  }
         else if (strcmp(argv[i], "-font") == 0 && i+1<argc) {
             i++;
             fname = argv[i];
         }
+        else if (strcmp(argv[i], "-grey") == 0)
+            background = 1;
         else if (strcmp(argv[i], "-term") == 0 && i+1<argc)
             termprog = argv[++i];
         else if (strcmp(argv[i], "-version") == 0) {
-            fprintf(stderr, "%s", version[0]);
+            fprintf(stderr, "%s", version[1]);
             if (PATCHLEVEL > 0)
                 fprintf(stderr, "; patch level %d", PATCHLEVEL);
             putc('\n', stderr);
             exit(0);
         }
+        else if (strcmp(argv[i], "-virtuals") == 0 && i+1<argc)
+	  {
+	    int n = atoi(argv[++i]);
+	    if (n > 0 && n <= 12)
+	      numvirtuals = n;
+	    else
+	      fprintf(stderr, "9wm: wrong number of virtual screens, must be between 1 and 12\n");
+	  }
         else if (argv[i][0] == '-')
             usage();
         else
@@ -120,7 +146,7 @@ char    *argv[];
     if (shell == NULL)
         shell = DEFSHELL;
 
-    dpy = XOpenDisplay("");
+    dpy = XOpenDisplay(display);
     if (dpy == 0)
         fatal("can't open display");
     screen = DefaultScreen(dpy);
@@ -202,8 +228,10 @@ char    *argv[];
 
     attr.cursor = arrow;
     attr.event_mask = SubstructureRedirectMask
-        | SubstructureNotifyMask | ColormapChangeMask
-        | ButtonPressMask | ButtonReleaseMask | KeyPressMask | PropertyChangeMask;
+      | SubstructureNotifyMask | ColormapChangeMask
+      | ButtonPressMask | ButtonReleaseMask
+      | PropertyChangeMask | StructureNotifyMask
+      | (use_keys ? KeyPressMask : 0) ;
     mask = CWCursor|CWEventMask;
     XChangeWindowAttributes(dpy, root, mask, &attr);
     XSync(dpy, False);
@@ -214,6 +242,7 @@ char    *argv[];
     }
 
     menuwin = XCreateSimpleWindow(dpy, root, 0, 0, 1, 1, 1, black, white);
+    initb2menu(numvirtuals);
 
     /* set selection so that 9term knows we're running */
     curtime = CurrentTime;
@@ -244,19 +273,36 @@ char    *argv[];
                 fprintf(stderr, "9wm: unknown ev.type %d\n", ev.type);
             break;
 	case KeyRelease:
-	  ev.xkey.window = current->window;
-	  XSendEvent(dpy, current->window, False, NoEventMask, &ev);
+	  if (use_keys && current)
+	    {
+	      ev.xkey.window = current->window;
+	      XSendEvent(dpy, current->window, False, NoEventMask, &ev);
+	    }
 	  break;
 	case KeyPress:
-	  if (XLookupKeysym(&(ev.xkey),0) == XK_Tab)
+	  if (use_keys)
 	    {
-	      activatenext();
+	      if (XLookupKeysym(&(ev.xkey),0) == XK_Tab && 
+		  (ev.xkey.state & ControlMask))
+		{
+		  if (ev.xkey.state & ShiftMask)
+		    activateprevious();
+		  else
+		    activatenext();		      
+		}
+	      else if (current)
+		{
+		  ev.xkey.window = current->window;
+		  XSendEvent(dpy, current->window, False, NoEventMask, &ev);
+		}
 	    }
-	  ev.xkey.window = current->window;
-	  XSendEvent(dpy, current->window, False, NoEventMask, &ev);
 	  break;
         case ButtonPress:
             button(&ev.xbutton);
+	    /*  option */
+	    {
+	      XAllowEvents (dpy, SyncPointer, ev.xbutton.time);
+	    }
             break;
         case ButtonRelease:
             break;
@@ -264,8 +310,8 @@ char    *argv[];
             mapreq(&ev.xmaprequest);
             break;
         case ConfigureRequest:
-            configurereq(&ev.xconfigurerequest);
-            break;
+	  configurereq(&ev.xconfigurerequest);
+	  break;
         case CirculateRequest:
             circulatereq(&ev.xcirculaterequest);
             break;
@@ -307,34 +353,74 @@ char    *argv[];
         case FocusIn:
         case FocusOut:
         case ConfigureNotify:
-        case MapNotify:
         case MappingNotify:
+        case MapNotify:
             /* not interested */
             break;
         }
     }
 }
 
+void
+activateprevious()
+{
+  Client * c, * tmp = NULL;
+
+  if (!current)
+    current = clients;
+
+  if (!clients || !current)
+    return;
+
+  for (c = clients; c->next; c=c->next)
+    {
+      if (c->virtual == virtual &&
+	  c->state == NormalState)
+	tmp = c;
+      if (tmp && 
+	  c->next == current)
+	break;
+    }
+
+  if (tmp && 
+      tmp->state == NormalState && 
+      tmp->parent != root)	/* paranoid */
+    {
+      active(tmp);
+      XMapRaised(dpy, tmp->parent);
+      return;
+    }
+}
+
 void
 activatenext()
 {
   Client * c;
 
-  for (c=current->next; c && c != current; c = ((c && c->next) ? c->next : clients))
-    if (c->virtual == virtual && 
+  if (!current)
+    current = clients;
+
+  if (!clients || !current)
+    return;
+
+  for (c=current->next; c != current; c = ((c && c->next) ? c->next : clients))
+    if (c &&
+	c->state == NormalState && 
+	c->virtual == virtual && 
 	c->parent != root)	/* paranoid */
       {
 	active(c);
 	XMapRaised(dpy, c->parent);
 	return;
       }
-
 }
 
 void
 usage()
 {
-    fprintf(stderr, "usage: 9wm [-grey] [-version] [-font fname] [-term prog] [exit|restart]\n");
+    fprintf(stderr, "usage: w9wm [[-display|-dpy] dpy] [-grey] [-version] [-font fname] [-pass]\n"
+"       [-nokeys] [-debug] [-nostalgia] [-term prog] [-pass] [-virtuals n]\n"
+"       [exit|restart]\n");
     exit(1);
 }
 
@@ -435,14 +521,21 @@ XConfigureRequestEvent *e;
     c = getclient(e->window, 0);
     if (c) {
         gravitate(c, 1);
+
         if (e->value_mask & CWX)
             c->x = e->x;
         if (e->value_mask & CWY)
             c->y = e->y;
         if (e->value_mask & CWWidth)
+	  {
             c->dx = e->width;
+	    XResizeWindow(dpy, c->parent, c->dx+2*(BORDER-1), c->dy+2*(BORDER-1)); 
+	  }
         if (e->value_mask & CWHeight)
+	  {
             c->dy = e->height;
+	    XResizeWindow(dpy, c->parent, c->dx+2*(BORDER-1), c->dy+2*(BORDER-1)); 
+	  }
         if (e->value_mask & CWBorderWidth)
             c->border = e->border_width;
         gravitate(c, 0);
@@ -504,7 +597,7 @@ XMapRequestEvent *e;
         XMapRaised(dpy, c->parent);
         XMapWindow(dpy, c->window);
         setstate9(c, NormalState);
-        if (c->trans != None && current && c->trans == current->window)
+	//        if (c->trans != None && current && c->trans == current->window)
                 active(c);
         break;
     case IconicState:
@@ -533,7 +626,9 @@ XUnmapEvent *e;
             if (c == current)
                 nofocus();
             if (!c->reparenting)
+	      {
                 withdraw(c);
+	      }
             break;
         }
         c->reparenting = 0;
