@@ -3,7 +3,7 @@
 #define BORDER      _border
 #define MAXHIDDEN   32
 #define B3FIXED     5
-#define NUMVIRTUALS 4
+#define NUMVIRTUALS 12
 
 #define AllButtonMask   (Button1Mask|Button2Mask|Button3Mask \
             |Button4Mask|Button5Mask)
@@ -83,6 +83,8 @@ extern int          initting;
 extern GC           gc;
 extern XFontStruct  *font;
 extern int          nostalgia;
+extern Atom	    exit_9wm;
+extern Atom	    restart_9wm;
 extern Atom         wm_state;
 extern Atom         _9wm_hold_mode;
 extern Atom         wm_protocols;
@@ -99,7 +101,8 @@ extern int          _border;
 extern int          min_cmaps;
 extern int          curtime;
 extern int          debug;
-
+extern int	    click_passes;
+extern int	    use_keys;
 /* client.c */
 extern Client       *clients;
 extern Client       *current;
@@ -110,7 +113,7 @@ extern int          numhidden;
 extern char         *b3items[];
 extern Menu         b3menu;
 extern int	    virtual;
-extern Client * currents[];
+extern Client *	    currents[];
 
 /* cursor.c */
 extern Cursor       target;
