@@ -1,5 +1,9 @@
 /* Copyright (c) 1994 David Hogan, 2000 Benjamin Drieu, see README for licence details */
 #include <stdio.h>
+#include <unistd.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -18,13 +22,22 @@ Client * currents[NUMVIRTUALS] =
 
 char    *b2items[NUMVIRTUALS+1] = 
 {
-    "One",
-    "Two",
-    " Three ",			/* without those spaces, b2 menu is ugly */
-    "Four",
-    0,
+  "  One  ",			/* to produce a nicer menu */
+  "Two",
+  "Three",
+  "Four",
+  "Five",
+  "Six",
+  "Seven",
+  "Eight",
+  "Nine",
+  "Ten",
+  "Eleven",
+  "Twelve",
+  0,
 };
 
+
 Menu    b2menu =
 {
     b2items,
@@ -55,7 +68,6 @@ void
 button(e)
 XButtonEvent *e;
 {
-    int n, shift;
     Client *c;
     Window dw;
 
@@ -73,18 +85,33 @@ XButtonEvent *e;
         if (c) {
             XMapRaised(dpy, c->parent);
             active(c);
+	    if (click_passes)
+	      XAllowEvents (dpy, ReplayPointer, curtime);
         }
         return;
     case Button2:
-        if ((e->state&(ShiftMask|ControlMask))==(ShiftMask|ControlMask))
-            menuhit(e, &egg);
-	else
-	  button2(e);
+        if (c && click_passes) {
+            XMapRaised(dpy, c->parent);
+            active(c);
+	    XAllowEvents (dpy, ReplayPointer, curtime);
+        }
+	else {
+            if ((e->state&(ShiftMask|ControlMask))==(ShiftMask|ControlMask))
+                menuhit(e, &egg);
+	    else
+	        button2(e);
+	}
         return;
     default:
         return;
     case Button3:
-        button3(e);
+        if (c && click_passes) {
+            XMapRaised(dpy, c->parent);
+            active(c);
+	    XAllowEvents (dpy, ReplayPointer, curtime);
+        }
+	else
+            button3(e);
         break;
     }
 }
@@ -94,7 +121,6 @@ void
 button2(e)
 XButtonEvent *e;
 {
-    Client *c;
     int n;
     cmapfocus(0);
     if ((n = menuhit(e, &b2menu)) == -1)
@@ -104,6 +130,12 @@ XButtonEvent *e;
         cmapfocus(current);
 }
 
+void
+initb2menu(n)
+int n;
+{
+  b2items[n] = 0;
+}
 
 void 
 button3(e)
@@ -147,7 +179,6 @@ void
 switch_to(n)
 int n;
 {
-  Client *c;
   if (n == virtual)
     return;
   currents[virtual] = current;
@@ -157,7 +188,6 @@ int n;
 }
 
 
-
 void
 switch_to_c(n,c)
 int n;
@@ -169,7 +199,7 @@ Client * c;
   if (c->parent == DefaultRootWindow(dpy))
     return;
 
-  if (c->virtual != virtual)
+  if (c->virtual != virtual && c->state == NormalState)
     {
       XUnmapWindow(dpy, c->parent);
       XUnmapWindow(dpy, c->window);
@@ -177,12 +207,14 @@ Client * c;
       if (c == current)
 	nofocus();
     }
-  else
+  else if (c->virtual == virtual &&  c->state == IconicState)
     {
       int i;
+
       for (i = 0; i < numhidden; i++)
 	if (c == hiddenc[i]) 
 	  break;
+
       if (i == numhidden)
 	{
 	  XMapWindow(dpy, c->window);
@@ -305,7 +337,7 @@ int map;
     }
     c = hiddenc[n];
     if (!hidden(c)) {
-        fprintf(stderr, "9wm: unhide: not hidden: %s(0x%x)\n",
+        fprintf(stderr, "9wm: unhide: not hidden: %s(0x%lx)\n",
             c->label, c->window);
         return;
     }
@@ -338,7 +370,7 @@ int map;
             unhide(i, map);
             return;
         }
-    fprintf(stderr, "9wm: unhidec: not hidden: %s(0x%x)\n",
+    fprintf(stderr, "9wm: unhidec: not hidden: %s(0x%lx)\n",
         c->label, c->window);
 }
 
